@@ -9,6 +9,7 @@ import atila
 from . import decorators, auth, template_engine, router, events
 from types import ModuleType
 import copy
+import inspect
 
 class MixIn (
         decorators.Decorators,
@@ -182,8 +183,13 @@ class MixIn (
             setattr (module, "__mntopt__", params)
             # for app initialzing and reloading
             self._mount_option = params
+
+            fspec = inspect.getfullargspec (mount_func)
             try:
-                mount_func (self)
+                if len (fspec.args) == 1:
+                    mount_func (self)
+                else:
+                    mount_func (self, params)
                 if 'point' in params:
                     self.log ("- {} mounted to {}".format (module.__name__, tc.white (self.mount_p [:-1] + params ['point'])), "info")
                 else:
@@ -325,8 +331,12 @@ class MixIn (
                 pass
             else:
                 self._current_mount_options = kargs
+                fspec = inspect.getfullargspec (setup_func)
                 try:
-                    setup_func (self, copy.copy (kargs))
+                    if len (fspec.args) == 1:
+                        setup_func (self)
+                    else:
+                        setup_func (self, copy.copy (kargs))
                 finally:
                     self._current_mount_options = None
 
