@@ -13,6 +13,10 @@ import skitai
 from skitai.exceptions import HTTPError
 import uuid
 
+def generate_otp (serial):
+    serial = int (md5 (str (serial).encode ()).hexdigest ()[-8:], 16)
+    return md5 (str (round (time.time () - serial, -1)).encode ()).hexdigest ()
+
 class WAS (wsgiappservice.WAS):
     # mehiods remap ------------------------------------------------
     def __getattr__ (self, name):
@@ -75,12 +79,30 @@ class WAS (wsgiappservice.WAS):
         return base64.encodebytes (md5 (s.encode ()).digest ()) [:-3].decode ().replace ('/', '-').replace ('+', '.')
 
     # JWT token --------------------------------------------------
-    def mkjwt (self, claim, salt = None, alg = "HS256"):
+    def encode_jwt (self, claim, salt = None, alg = "HS256"):
         assert "exp" in claim, "exp claim required"
         return jwt.gen_token (salt or self.app.salt, claim, alg)
 
-    def dejwt (self, token = None, salt = None):
+    def decode_jwt (self, token = None, salt = None):
         return self.request.dejwt (token, salt or self.app.salt)
+    mkjwt = encode_jwt
+    dejwt = decode_jwt
+
+    # otp ---------------------------------------------------
+    def generate_otp (self, salt = None):
+        return generate_otp (salt or self.app.salt)
+
+    def verify_otp (self, otp, salt = None):
+        now = time.time ()
+        salt = salt or self.app.salt
+        serial = int (md5 (str (salt).encode ()).hexdigest ()[-8:], 16)
+        if otp == md5 (str (round (now - serial, -1)).encode ()).hexdigest ():
+            return True
+        if otp == md5 (str (round (now + 10 - serial, -1)).encode ()).hexdigest ():
+            return True
+        if otp == md5 (str (round (now - 10 - serial, -1)).encode ()).hexdigest ():
+            return True
+        return False
 
     # onetime token  ----------------------------------------
     def _unserialize_token (self, string):
@@ -103,12 +125,13 @@ class WAS (wsgiappservice.WAS):
             return
         return pickle.loads (data)
 
-    def mkott (self, obj, timeout = 1200, session_key = None):
+    def encode_ott (self, obj, timeout = 1200, session_key = None):
         wrapper = {
             'object': obj,
-            'timeout': time.time () + timeout
+            'timeout': timeout and time.time () + timeout or 0
         }
         if session_key:
+            assert timeout, 'session ott require timeout'
             token = hex (random.getrandbits (64))
             tokey = '_{}_token'.format (session_key)
             wrapper ['_session_token'] = (tokey, token)
@@ -121,7 +144,7 @@ class WAS (wsgiappservice.WAS):
         mac.update (data)
         return (base64.b64encode (mac.digest()).strip().rstrip (b'=') + b"?" + base64.b64encode (data).strip ().rstrip (b'=')).decode ("utf8")
 
-    def deott (self, string):
+    def decode_ott (self, string):
         wrapper = self._unserialize_token (string)
         if not wrapper:
             return
@@ -129,7 +152,8 @@ class WAS (wsgiappservice.WAS):
         # validation with session
         tokey = None
         has_error = False
-        if wrapper ['timeout']  < time.time ():
+        timeout = wrapper ['timeout']
+        if timeout and timeout  < time.time ():
             has_error = True
         if not has_error:
             session_token = wrapper.get ('_session_token')
@@ -150,7 +174,7 @@ class WAS (wsgiappservice.WAS):
         obj = wrapper ['object']
         return obj
 
-    def rvott (self, string):
+    def revoke_ott (self, string):
         # revoke token
         wrapper = self._unserialize_token (string)
         session_token = wrapper.get ('_session_token')
@@ -166,9 +190,9 @@ class WAS (wsgiappservice.WAS):
         self.session.expire ()
         self.session.mount ()
 
-    mktoken = token = mkott
-    rmtoken = rvott
-    detoken = deott
+    mktoken = token = mkott = encode_ott
+    rmtoken = rvott = revoke_ott
+    detoken = deott = decode_ott
 
     # CSRF token ------------------------------------------------------
     CSRF_NAME = "XSRF_TOKEN" #axios compat
